@@ -16,6 +16,8 @@ import (
 	"github.com/aquasecurity/trivy/pkg/commands/option"
 	"github.com/aquasecurity/trivy/pkg/commands/plugin"
 	"github.com/aquasecurity/trivy/pkg/commands/server"
+	"github.com/aquasecurity/trivy/pkg/k8s"
+	"github.com/aquasecurity/trivy/pkg/log"
 	"github.com/aquasecurity/trivy/pkg/result"
 	"github.com/aquasecurity/trivy/pkg/types"
 	"github.com/aquasecurity/trivy/pkg/utils"
@@ -147,8 +149,8 @@ var (
 
 	securityChecksFlag = cli.StringFlag{
 		Name:    "security-checks",
-		Value:   types.SecurityCheckVulnerability,
-		Usage:   "comma-separated list of what security issues to detect (vuln,config)",
+		Value:   fmt.Sprintf("%s,%s", types.SecurityCheckVulnerability, types.SecurityCheckSecret),
+		Usage:   "comma-separated list of what security issues to detect (vuln,config,secret)",
 		EnvVars: []string{"TRIVY_SECURITY_CHECKS"},
 	}
 
@@ -166,6 +168,12 @@ var (
 		EnvVars: []string{"TRIVY_CACHE_BACKEND"},
 	}
 
+	cacheTTL = cli.DurationFlag{
+		Name:    "cache-ttl",
+		Usage:   "cache TTL when using redis as cache backend",
+		EnvVars: []string{"TRIVY_CACHE_TTL"},
+	}
+
 	redisBackendCACert = cli.StringFlag{
 		Name:    "redis-ca",
 		Usage:   "redis ca file location, if using redis as cache backend",
@@ -201,6 +209,20 @@ var (
 		EnvVars: []string{"TRIVY_TIMEOUT"},
 	}
 
+	namespaceFlag = cli.StringFlag{
+		Name:    "namespace",
+		Aliases: []string{"n"},
+		Value:   "",
+		Usage:   "specify a namespace to scan",
+		EnvVars: []string{"TRIVY_K8S_NAMESPACE"},
+	}
+
+	reportFlag = cli.StringFlag{
+		Name:  "report",
+		Value: "all",
+		Usage: "specify a report format for the output. (all,summary default: all)",
+	}
+
 	// TODO: remove this flag after a sufficient deprecation period.
 	lightFlag = cli.BoolFlag{
 		Name:    "light",
@@ -332,6 +354,13 @@ var (
 		EnvVars: []string{"TRIVY_DB_REPOSITORY"},
 	}
 
+	secretConfig = cli.StringFlag{
+		Name:    "secret-config",
+		Usage:   "specify a path to config file for secret scanning",
+		Value:   "trivy-secret.yaml",
+		EnvVars: []string{"TRIVY_SECRET_CONFIG"},
+	}
+
 	// Global flags
 	globalFlags = []cli.Flag{
 		&quietFlag,
@@ -350,7 +379,7 @@ func NewApp(version string) *cli.App {
 	app.Name = "trivy"
 	app.Version = version
 	app.ArgsUsage = "target"
-	app.Usage = "A simple and comprehensive vulnerability scanner for containers"
+	app.Usage = "Scanner for vulnerabilities in container images, file systems, and Git repositories, as well as for configuration issues and hard-coded secrets"
 	app.EnableBashCompletion = true
 	app.Flags = globalFlags
 
@@ -372,12 +401,13 @@ func NewApp(version string) *cli.App {
 		NewImageCommand(),
 		NewFilesystemCommand(),
 		NewRootfsCommand(),
-		NewSbomCommand(),
 		NewRepositoryCommand(),
 		NewClientCommand(),
 		NewServerCommand(),
 		NewConfigCommand(),
 		NewPluginCommand(),
+		NewK8sCommand(),
+		NewSbomCommand(),
 		NewVersionCommand(),
 	}
 	app.Commands = append(app.Commands, plugin.LoadCommands()...)
@@ -450,14 +480,22 @@ func NewImageCommand() *cli.Command {
 			&ignorePolicy,
 			&listAllPackages,
 			&cacheBackendFlag,
+			&cacheTTL,
 			&redisBackendCACert,
 			&redisBackendCert,
 			&redisBackendKey,
 			&offlineScan,
 			&insecureFlag,
 			&dbRepositoryFlag,
+			&secretConfig,
 			stringSliceFlag(skipFiles),
 			stringSliceFlag(skipDirs),
+
+			// for client/server
+			&remoteServer,
+			&token,
+			&tokenHeader,
+			&customHeaders,
 		},
 	}
 }
@@ -484,6 +522,7 @@ func NewFilesystemCommand() *cli.Command {
 			&securityChecksFlag,
 			&ignoreFileFlag,
 			&cacheBackendFlag,
+			&cacheTTL,
 			&redisBackendCACert,
 			&redisBackendCert,
 			&redisBackendKey,
@@ -493,6 +532,7 @@ func NewFilesystemCommand() *cli.Command {
 			&listAllPackages,
 			&offlineScan,
 			&dbRepositoryFlag,
+			&secretConfig,
 			stringSliceFlag(skipFiles),
 			stringSliceFlag(skipDirs),
 
@@ -531,6 +571,7 @@ func NewRootfsCommand() *cli.Command {
 			&securityChecksFlag,
 			&ignoreFileFlag,
 			&cacheBackendFlag,
+			&cacheTTL,
 			&redisBackendCACert,
 			&redisBackendCert,
 			&redisBackendKey,
@@ -539,6 +580,8 @@ func NewRootfsCommand() *cli.Command {
 			&ignorePolicy,
 			&listAllPackages,
 			&offlineScan,
+			&dbRepositoryFlag,
+			&secretConfig,
 			stringSliceFlag(skipFiles),
 			stringSliceFlag(skipDirs),
 			stringSliceFlag(configPolicy),
@@ -572,6 +615,7 @@ func NewRepositoryCommand() *cli.Command {
 			&securityChecksFlag,
 			&ignoreFileFlag,
 			&cacheBackendFlag,
+			&cacheTTL,
 			&redisBackendCACert,
 			&redisBackendCert,
 			&redisBackendKey,
@@ -582,6 +626,8 @@ func NewRepositoryCommand() *cli.Command {
 			&listAllPackages,
 			&offlineScan,
 			&insecureFlag,
+			&dbRepositoryFlag,
+			&secretConfig,
 			stringSliceFlag(skipFiles),
 			stringSliceFlag(skipDirs),
 		},
@@ -594,8 +640,12 @@ func NewClientCommand() *cli.Command {
 		Name:      "client",
 		Aliases:   []string{"c"},
 		ArgsUsage: "image_name",
-		Usage:     "client mode",
-		Action:    artifact.ImageRun,
+		Usage:     "[DEPRECATED] client mode",
+		Action: func(ctx *cli.Context) error {
+			log.Logger.Warn("`client` subcommand is deprecated now. See https://github.com/aquasecurity/trivy/discussions/2119")
+			return artifact.ImageRun(ctx)
+		},
+		Hidden: true, // It is no longer displayed
 		Flags: []cli.Flag{
 			&templateFlag,
 			&formatFlag,
@@ -618,6 +668,7 @@ func NewClientCommand() *cli.Command {
 			&listAllPackages,
 			&offlineScan,
 			&insecureFlag,
+			&secretConfig,
 
 			&token,
 			&tokenHeader,
@@ -646,9 +697,11 @@ func NewServerCommand() *cli.Command {
 			&downloadDBOnlyFlag,
 			&resetFlag,
 			&cacheBackendFlag,
+			&cacheTTL,
 			&redisBackendCACert,
 			&redisBackendCert,
 			&redisBackendKey,
+			&dbRepositoryFlag,
 
 			// original flags
 			&token,
@@ -745,6 +798,65 @@ func NewPluginCommand() *cli.Command {
 	}
 }
 
+// NewK8sCommand is the factory method to add k8s subcommand
+func NewK8sCommand() *cli.Command {
+	k8sSecurityChecksFlag := withValue(
+		securityChecksFlag,
+		fmt.Sprintf("%s,%s", types.SecurityCheckVulnerability, types.SecurityCheckConfig),
+	)
+
+	return &cli.Command{
+		Name:    "kubernetes",
+		Aliases: []string{"k8s"},
+		Usage:   "scan kubernetes vulnerabilities and misconfigurations",
+		CustomHelpTemplate: cli.CommandHelpTemplate + `EXAMPLES:
+  - cluster scanning:
+      $ trivy k8s --report summary
+
+  - namespace scanning:
+      $ trivy k8s -n kube-system --report summary
+
+  - resource scanning:
+      $ trivy k8s deployment/orion
+`,
+		Action: k8s.Run,
+		Flags: []cli.Flag{
+			&namespaceFlag,
+			&reportFlag,
+			&formatFlag,
+			&outputFlag,
+			&severityFlag,
+			&exitCodeFlag,
+			&skipDBUpdateFlag,
+			&skipPolicyUpdateFlag,
+			&clearCacheFlag,
+			&ignoreUnfixedFlag,
+			&vulnTypeFlag,
+			&k8sSecurityChecksFlag,
+			&ignoreFileFlag,
+			&cacheBackendFlag,
+			&cacheTTL,
+			&redisBackendCACert,
+			&redisBackendCert,
+			&redisBackendKey,
+			&timeoutFlag,
+			&noProgressFlag,
+			&ignorePolicy,
+			&listAllPackages,
+			&offlineScan,
+			&dbRepositoryFlag,
+			&secretConfig,
+			stringSliceFlag(skipFiles),
+			stringSliceFlag(skipDirs),
+
+			// for misconfiguration
+			stringSliceFlag(configPolicy),
+			stringSliceFlag(configData),
+			stringSliceFlag(policyNamespaces),
+		},
+	}
+}
+
 // NewSbomCommand is the factory method to add sbom command
 func NewSbomCommand() *cli.Command {
 	return &cli.Command{
@@ -774,6 +886,7 @@ func NewSbomCommand() *cli.Command {
 			&timeoutFlag,
 			&severityFlag,
 			&offlineScan,
+			&dbRepositoryFlag,
 			stringSliceFlag(skipFiles),
 			stringSliceFlag(skipDirs),
 
@@ -789,7 +902,7 @@ func NewSbomCommand() *cli.Command {
 				Name:    "sbom-format",
 				Aliases: []string{"format"},
 				Value:   "cyclonedx",
-				Usage:   "SBOM format (cyclonedx)",
+				Usage:   "SBOM format (cyclonedx, spdx, spdx-json)",
 				EnvVars: []string{"TRIVY_SBOM_FORMAT"},
 			},
 		},
@@ -817,3 +930,8 @@ func NewVersionCommand() *cli.Command {
 func stringSliceFlag(f cli.StringSliceFlag) *cli.StringSliceFlag {
 	return &f
 }
+
+func withValue(s cli.StringFlag, value string) cli.StringFlag {
+	s.Value = value
+	return s
+}
