@@ -1,8 +1,6 @@
 package types
 
 import (
-	"time"
-
 	"github.com/caarlos0/env/v6"
 	"golang.org/x/xerrors"
 
@@ -14,12 +12,11 @@ type DockerConfig struct {
 	UserName      string `env:"TRIVY_USERNAME"`
 	Password      string `env:"TRIVY_PASSWORD"`
 	RegistryToken string `env:"TRIVY_REGISTRY_TOKEN"`
-	Insecure      bool   `env:"TRIVY_INSECURE" envDefault:"false"`
 	NonSSL        bool   `env:"TRIVY_NON_SSL" envDefault:"false"`
 }
 
 // GetDockerOption returns the Docker scanning options using DockerConfig
-func GetDockerOption(timeout time.Duration) (types.DockerOption, error) {
+func GetDockerOption(insecureTlsSkip bool) (types.DockerOption, error) {
 	cfg := DockerConfig{}
 	if err := env.Parse(&cfg); err != nil {
 		return types.DockerOption{}, xerrors.Errorf("unable to parse environment variables: %w", err)
@@ -29,8 +26,7 @@ func GetDockerOption(timeout time.Duration) (types.DockerOption, error) {
 		UserName:              cfg.UserName,
 		Password:              cfg.Password,
 		RegistryToken:         cfg.RegistryToken,
-		Timeout:               timeout,
-		InsecureSkipTLSVerify: cfg.Insecure,
+		InsecureSkipTLSVerify: insecureTlsSkip,
 		NonSSL:                cfg.NonSSL,
 	}, nil
 }
